@@ -19,10 +19,23 @@
 package davmail;
 
 import davmail.ui.tray.DavGatewayTray;
-import org.apache.log4j.*;
+import org.apache.log4j.ConsoleAppender;
+import org.apache.log4j.FileAppender;
+import org.apache.log4j.Level;
+import org.apache.log4j.Logger;
+import org.apache.log4j.PatternLayout;
+import org.apache.log4j.RollingFileAppender;
 
-import java.io.*;
-import java.util.*;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.Enumeration;
+import java.util.Iterator;
+import java.util.Map;
+import java.util.Properties;
+import java.util.TreeSet;
 
 /**
  * Settings facade.
@@ -35,6 +48,7 @@ public final class Settings {
     public static final String O365 = "O365";
     public static final String O365_MODERN = "O365Modern";
     public static final String O365_INTERACTIVE = "O365Interactive";
+    public static final String O365_MANUAL = "O365Manual";
     public static final String WEBDAV = "WebDav";
     public static final String EWS = "EWS";
     public static final String AUTO = "Auto";
@@ -169,6 +183,7 @@ public final class Settings {
         SETTINGS.put("davmail.forceActiveSyncUpdate", Boolean.FALSE.toString());
         SETTINGS.put("davmail.showStartupBanner", Boolean.TRUE.toString());
         SETTINGS.put("davmail.disableGuiNotifications", Boolean.FALSE.toString());
+        SETTINGS.put("davmail.disableTrayActivitySwitch", Boolean.FALSE.toString());
         SETTINGS.put("davmail.imapAutoExpunge", Boolean.TRUE.toString());
         SETTINGS.put("davmail.imapAlwaysApproxMsgSize", Boolean.FALSE.toString());
         SETTINGS.put("davmail.popMarkReadOnRetr", Boolean.FALSE.toString());
@@ -307,6 +322,9 @@ public final class Settings {
         Settings.setLoggingLevel("davmail", Settings.getLoggingLevel("davmail"));
         Settings.setLoggingLevel("httpclient.wire", Settings.getLoggingLevel("httpclient.wire"));
         Settings.setLoggingLevel("org.apache.commons.httpclient", Settings.getLoggingLevel("org.apache.commons.httpclient"));
+        // set logging levels for HttpClient 4
+        Settings.setLoggingLevel("org.apache.http.wire", Settings.getLoggingLevel("httpclient.wire"));
+        Settings.setLoggingLevel("org.apache.http", Settings.getLoggingLevel("org.apache.commons.httpclient"));
     }
 
     /**
@@ -558,7 +576,13 @@ public final class Settings {
         return System.getProperty("os.name").toLowerCase().startsWith("linux");
     }
 
+    public static boolean isUnix() {
+        return isLinux() ||
+                System.getProperty("os.name").toLowerCase().startsWith("freebsd");
+    }
+
     public static boolean isJava8() {
         return "1.8".equals(System.getProperty("java.specification.version"));
     }
+
 }
