@@ -22,26 +22,33 @@ import davmail.AbstractConnection;
 import davmail.BundleMessage;
 import davmail.DavGateway;
 import davmail.Settings;
-import davmail.exception.*;
+import davmail.exception.DavMailAuthenticationException;
+import davmail.exception.DavMailException;
+import davmail.exception.HttpNotFoundException;
+import davmail.exception.HttpPreconditionFailedException;
+import davmail.exception.HttpServerErrorException;
 import davmail.exchange.ExchangeSession;
 import davmail.exchange.ExchangeSessionFactory;
 import davmail.exchange.ICSBufferedReader;
 import davmail.exchange.XMLStreamUtil;
 import davmail.exchange.dav.DavExchangeSession;
+import davmail.http.URIUtil;
 import davmail.ui.tray.DavGatewayTray;
 import davmail.util.IOUtil;
 import davmail.util.StringUtil;
 import org.apache.commons.httpclient.HttpException;
-import org.apache.commons.httpclient.HttpStatus;
-import org.apache.commons.httpclient.URI;
-import org.apache.commons.httpclient.URIException;
-import org.apache.commons.httpclient.util.URIUtil;
+import org.apache.http.HttpStatus;
+import org.apache.http.impl.EnglishReasonPhraseCatalog;
 import org.apache.log4j.Logger;
 
-import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
-import java.io.*;
+import java.io.BufferedOutputStream;
+import java.io.IOException;
+import java.io.OutputStream;
+import java.io.OutputStreamWriter;
+import java.io.StringReader;
+import java.io.Writer;
 import java.net.Socket;
 import java.net.SocketException;
 import java.net.SocketTimeoutException;
@@ -66,15 +73,15 @@ public class CaldavConnection extends AbstractConnection {
      */
     public static final BitSet ical_allowed_abs_path = new BitSet(256);
     static {
-        ical_allowed_abs_path.or(URI.allowed_abs_path);
+        ical_allowed_abs_path.or(URIUtil.allowed_abs_path);
         ical_allowed_abs_path.clear('@');
     }
 
-    static String encodePath(CaldavRequest request, String path) throws URIException {
+    static String encodePath(CaldavRequest request, String path) {
         if (request.isIcal5()) {
-            return URIUtil.encode(path, ical_allowed_abs_path, "UTF-8");
+            return URIUtil.encode(path, ical_allowed_abs_path);
         } else {
-            return URIUtil.encodePath(path, "UTF-8");
+            return URIUtil.encodePath(path);
         }
     }
 
@@ -234,7 +241,7 @@ public class CaldavConnection extends AbstractConnection {
         } else if (request.isPath(1, ".well-known")) {
             sendWellKnown();
         } else {
-            sendUnsupported(request);
+            sendNotFound(request);
         }
     }
 
@@ -249,7 +256,7 @@ public class CaldavConnection extends AbstractConnection {
             } else if (request.isPropFind() && request.isPathLength(3)) {
                 sendPrincipalsFolder(request);
             } else {
-                sendUnsupported(request);
+                sendNotFound(request);
             }
         } else if (request.isPath(2, "public")) {
             StringBuilder prefixBuffer = new StringBuilder("public");
@@ -258,7 +265,7 @@ public class CaldavConnection extends AbstractConnection {
             }
             sendPrincipal(request, URIUtil.decode(prefixBuffer.toString()), URIUtil.decode(request.getLastPath()));
         } else {
-            sendUnsupported(request);
+            sendNotFound(request);
         }
     }
 
@@ -288,7 +295,7 @@ public class CaldavConnection extends AbstractConnection {
             String etag = request.getHeader("if-match");
             String noneMatch = request.getHeader("if-none-match");
             ExchangeSession.ItemResult itemResult = session.createOrUpdateItem(request.getFolderPath(), lastPath, request.getBody(), etag, noneMatch);
-            sendHttpResponse(itemResult.status, buildEtagHeader(itemResult.etag), null, "", true);
+            sendHttpResponse(itemResult.status, buildEtagHeader(request, itemResult), null, "", true);
 
         } else if (request.isDelete()) {
             if (request.getFolderPath().endsWith("inbox")) {
@@ -359,11 +366,27 @@ public class CaldavConnection extends AbstractConnection {
             session.moveItem(request.path, URIUtil.decode(new URL(destinationUrl).getPath()));
             sendHttpResponse(HttpStatus.SC_CREATED, null);
         } else {
-            sendUnsupported(request);
+            sendNotFound(request);
         }
 
     }
 
+    protected HashMap<String, String> buildEtagHeader(CaldavRequest request, ExchangeSession.ItemResult itemResult) {
+        HashMap<String, String> headers = null;
+        if (itemResult.etag != null) {
+            headers = new HashMap<String, String>();
+            headers.put("ETag", itemResult.etag);
+        }
+        if (itemResult.itemName != null) {
+            if (headers == null) {
+                headers = new HashMap<String, String>();
+            }
+            headers.put("Location", buildEventPath(request, itemResult.itemName));
+        }
+        return headers;
+    }
+
+
     protected HashMap<String, String> buildEtagHeader(String etag) {
         if (etag != null) {
             HashMap<String, String> etagHeader = new HashMap<String, String>();
@@ -375,12 +398,13 @@ public class CaldavConnection extends AbstractConnection {
     }
 
     private void appendContactsResponses(CaldavResponse response, CaldavRequest request, List<ExchangeSession.Contact> contacts) throws IOException {
-        int size = contacts.size();
-        int count = 0;
-        for (ExchangeSession.Contact contact : contacts) {
-            DavGatewayTray.debug(new BundleMessage("LOG_LISTING_ITEM", ++count, size));
-            DavGatewayTray.switchIcon();
-            appendItemResponse(response, request, contact);
+        if (contacts != null) {
+            int count = 0;
+            for (ExchangeSession.Contact contact : contacts) {
+                DavGatewayTray.debug(new BundleMessage("LOG_LISTING_ITEM", ++count, contacts.size()));
+                DavGatewayTray.switchIcon();
+                appendItemResponse(response, request, contact);
+            }
         }
     }
 
@@ -394,15 +418,18 @@ public class CaldavConnection extends AbstractConnection {
         }
     }
 
-    protected void appendItemResponse(CaldavResponse response, CaldavRequest request, ExchangeSession.Item item) throws IOException {
+    protected String buildEventPath(CaldavRequest request, String itemName) {
         StringBuilder eventPath = new StringBuilder();
-        eventPath.append(encodePath(request, request.getPath()));
+        eventPath.append(encodePath(request, request.getFolderPath()));
         if (!(eventPath.charAt(eventPath.length() - 1) == '/')) {
             eventPath.append('/');
         }
-        String itemName = StringUtil.xmlEncode(item.getName());
-        eventPath.append(URIUtil.encodeWithinQuery(itemName));
-        response.startResponse(eventPath.toString());
+        eventPath.append(URIUtil.encodeWithinQuery(StringUtil.xmlEncode(itemName)));
+        return eventPath.toString();
+    }
+
+    protected void appendItemResponse(CaldavResponse response, CaldavRequest request, ExchangeSession.Item item) throws IOException {
+        response.startResponse(buildEventPath(request, item.getName()));
         response.startPropstat();
         if (request.hasProperty("calendar-data") && item instanceof ExchangeSession.Event) {
             response.appendCalendarData(item.getBody());
@@ -424,7 +451,7 @@ public class CaldavConnection extends AbstractConnection {
             response.appendProperty("D:resourcetype");
         }
         if (request.hasProperty("displayname")) {
-            response.appendProperty("D:displayname", itemName);
+            response.appendProperty("D:displayname", item.getName());
         }
         response.endPropStatOK();
         response.endResponse();
@@ -590,9 +617,9 @@ public class CaldavConnection extends AbstractConnection {
      * @throws IOException on error
      */
     public void sendGetRoot() throws IOException {
-        String buffer = "Connected to DavMail<br/>" +
-                "UserName :" + userName + "<br/>" +
-                "Email :" + session.getEmail() + "<br/>";
+        String buffer = "Connected to DavMail"+DavGateway.getCurrentVersion()+"<br/>" +
+                "UserName: " + userName + "<br/>" +
+                "Email: " + session.getEmail() + "<br/>";
         sendHttpResponse(HttpStatus.SC_OK, null, "text/html;charset=UTF-8", buffer, true);
     }
 
@@ -1096,12 +1123,12 @@ public class CaldavConnection extends AbstractConnection {
                     .append("PRODID:-//davmail.sf.net/NONSGML DavMail Calendar V1.1//EN").append((char) 13).append((char) 10)
                     .append("METHOD:REPLY").append((char) 13).append((char) 10)
                     .append("BEGIN:VFREEBUSY").append((char) 13).append((char) 10)
-                    .append("DTSTAMP:").append(valueMap.get("DTSTAMP")).append("").append((char) 13).append((char) 10)
-                    .append("ORGANIZER:").append(valueMap.get("ORGANIZER")).append("").append((char) 13).append((char) 10)
-                    .append("DTSTART:").append(valueMap.get("DTSTART")).append("").append((char) 13).append((char) 10)
-                    .append("DTEND:").append(valueMap.get("DTEND")).append("").append((char) 13).append((char) 10)
-                    .append("UID:").append(valueMap.get("UID")).append("").append((char) 13).append((char) 10)
-                    .append(attendeeKeyMap.get(attendee)).append(':').append(attendee).append("").append((char) 13).append((char) 10);
+                    .append("DTSTAMP:").append(valueMap.get("DTSTAMP")).append((char) 13).append((char) 10)
+                    .append("ORGANIZER:").append(valueMap.get("ORGANIZER")).append((char) 13).append((char) 10)
+                    .append("DTSTART:").append(valueMap.get("DTSTART")).append((char) 13).append((char) 10)
+                    .append("DTEND:").append(valueMap.get("DTEND")).append((char) 13).append((char) 10)
+                    .append("UID:").append(valueMap.get("UID")).append((char) 13).append((char) 10)
+                    .append(attendeeKeyMap.get(attendee)).append(':').append(attendee).append((char) 13).append((char) 10);
             entry.getValue().appendTo(ics);
             ics.append("END:VFREEBUSY").append((char) 13).append((char) 10)
                     .append("END:VCALENDAR");
@@ -1142,15 +1169,15 @@ public class CaldavConnection extends AbstractConnection {
     }
 
     /**
-     * Send 400 bad response for unsupported request.
+     * Send 404 not found for unknown request.
      *
      * @param request Caldav request
      * @throws IOException on error
      */
-    public void sendUnsupported(CaldavRequest request) throws IOException {
+    public void sendNotFound(CaldavRequest request) throws IOException {
         BundleMessage message = new BundleMessage("LOG_UNSUPPORTED_REQUEST", request);
-        DavGatewayTray.error(message);
-        sendErr(HttpStatus.SC_BAD_REQUEST, message.format());
+        DavGatewayTray.warn(message);
+        sendErr(HttpStatus.SC_NOT_FOUND, message.format());
     }
 
     /**
@@ -1247,7 +1274,7 @@ public class CaldavConnection extends AbstractConnection {
      * @throws IOException on error
      */
     public void sendHttpResponse(int status, Map<String, String> headers, String contentType, byte[] content, boolean keepAlive) throws IOException {
-        sendClient("HTTP/1.1 " + status + ' ' + HttpStatus.getStatusText(status));
+        sendClient("HTTP/1.1 " + status + ' ' + EnglishReasonPhraseCatalog.INSTANCE.getReason(status, Locale.ENGLISH));
         if (status != HttpStatus.SC_UNAUTHORIZED) {
             sendClient("Server: DavMail Gateway " + DavGateway.getCurrentVersion());
             String scheduleMode;
@@ -1476,11 +1503,11 @@ public class CaldavConnection extends AbstractConnection {
 
         protected boolean isUserAgent(String key) {
             String userAgent = headers.get("user-agent");
-            return userAgent != null && userAgent.indexOf(key) >= 0;
+            return userAgent != null && userAgent.contains(key);
         }
 
         public boolean isFreeBusy() {
-            return body != null && body.indexOf("VFREEBUSY") >= 0;
+            return body != null && body.contains("VFREEBUSY");
         }
 
         protected void buildDepth() {
@@ -1551,12 +1578,8 @@ public class CaldavConnection extends AbstractConnection {
             }
         }
 
-        protected boolean isEndTag(XMLStreamReader reader, String tagLocalName) {
-            return (reader.getEventType() == XMLStreamConstants.END_ELEMENT) && (reader.getLocalName().equals(tagLocalName));
-        }
-
         public void handleCompFilter(XMLStreamReader reader) throws XMLStreamException {
-            while (reader.hasNext() && !isEndTag(reader, "comp-filter")) {
+            while (reader.hasNext() && !XMLStreamUtil.isEndTag(reader, "comp-filter")) {
                 reader.next();
                 if (XMLStreamUtil.isStartTag(reader, "comp-filter")) {
                     String name = reader.getAttributeValue(null, "name");
@@ -1573,7 +1596,7 @@ public class CaldavConnection extends AbstractConnection {
         }
 
         public void handleProp(XMLStreamReader reader) throws XMLStreamException {
-            while (reader.hasNext() && !isEndTag(reader, "prop")) {
+            while (reader.hasNext() && !XMLStreamUtil.isEndTag(reader, "prop")) {
                 reader.next();
                 if (XMLStreamUtil.isStartTag(reader)) {
                     String tagLocalName = reader.getLocalName();
@@ -1691,15 +1714,13 @@ public class CaldavConnection extends AbstractConnection {
                     sendClient(Integer.toHexString(length));
                     sendClient(data, offset, length);
                     if (wireLogger.isDebugEnabled()) {
-                        StringBuilder logBuffer = new StringBuilder("> ");
-                        logBuffer.append(new String(data, offset, length, "UTF-8"));
-                        wireLogger.debug(logBuffer.toString());
+                        wireLogger.debug("> " + new String(data, offset, length, "UTF-8"));
                     }
                     sendClient("");
                 }
 
                 @Override
-                public void write(int b) throws IOException {
+                public void write(int b) {
                     throw new UnsupportedOperationException();
                 }
 
@@ -1779,28 +1800,27 @@ public class CaldavConnection extends AbstractConnection {
 
         public void appendProperty(String propertyName, String namespace, String propertyValue) throws IOException {
             if (propertyValue != null) {
-                writer.write('<');
-                writer.write(propertyName);
-                if (namespace != null) {
-                    writer.write("  xmlns:");
-                    writer.write(namespace);
-                }
+                startTag(propertyName, namespace);
                 writer.write('>');
                 writer.write(propertyValue);
                 writer.write("</");
                 writer.write(propertyName);
                 writer.write('>');
             } else {
-                writer.write('<');
-                writer.write(propertyName);
-                if (namespace != null) {
-                    writer.write("  xmlns:");
-                    writer.write(namespace);
-                }
+                startTag(propertyName, namespace);
                 writer.write("/>");
             }
         }
 
+        private void startTag(String propertyName, String namespace) throws IOException {
+            writer.write('<');
+            writer.write(propertyName);
+            if (namespace != null) {
+                writer.write(" xmlns:");
+                writer.write(namespace);
+            }
+        }
+
         public void endPropStatOK() throws IOException {
             writer.write("</D:prop><D:status>HTTP/1.1 200 OK</D:status></D:propstat>");
         }
