@@ -24,6 +24,7 @@ import davmail.exception.DavMailAuthenticationException;
 import davmail.exception.DavMailException;
 import davmail.exception.WebdavNotAvailableException;
 import davmail.exchange.auth.ExchangeAuthenticator;
+import davmail.exchange.auth.ExchangeFormAuthenticator;
 import davmail.exchange.dav.DavExchangeSession;
 import davmail.exchange.ews.EwsExchangeSession;
 import davmail.http.DavGatewayHttpClientFacade;
@@ -120,6 +121,11 @@ public final class ExchangeSessionFactory {
     public static ExchangeSession getInstance(String baseUrl, String userName, String password) throws IOException {
         ExchangeSession session = null;
         try {
+            String mode = Settings.getProperty("davmail.mode");
+            if (Settings.O365.equals(mode)) {
+                // force url wit O365
+                baseUrl = Settings.O365_URL;
+            }
 
             PoolKey poolKey = new PoolKey(baseUrl, userName, password);
 
@@ -141,7 +147,6 @@ public final class ExchangeSessionFactory {
             }
 
             if (session == null) {
-                String mode = Settings.getProperty("davmail.mode");
                 // convert old setting
                 if (mode == null) {
                     if ("false".equals(Settings.getProperty("davmail.enableEws"))) {
@@ -160,44 +165,62 @@ public final class ExchangeSessionFactory {
                         if (GraphicsEnvironment.isHeadless()) {
                             throw new DavMailException("EXCEPTION_DAVMAIL_CONFIGURATION", "O365Interactive not supported in headless mode");
                         }
+                    } else if (Settings.O365_MANUAL.equals(mode)) {
+                        authenticatorClass = "davmail.exchange.auth.O365ManualAuthenticator";
+                        if (GraphicsEnvironment.isHeadless()) {
+                            throw new DavMailException("EXCEPTION_DAVMAIL_CONFIGURATION", "O365Manual not supported in headless mode");
+                        }
                     }
                 }
 
                 if (authenticatorClass != null) {
                     ExchangeAuthenticator authenticator = (ExchangeAuthenticator) Class.forName(authenticatorClass).newInstance();
-                    authenticator.setUsername(userName);
-                    authenticator.setPassword(password);
+                    authenticator.setUsername(poolKey.userName);
+                    authenticator.setPassword(poolKey.password);
                     authenticator.authenticate();
-                    HttpClient httpClient = DavGatewayHttpClientFacade.getInstance(authenticator.getEWSUrl());
-                    session = new EwsExchangeSession(httpClient, userName);
-                    ((EwsExchangeSession) session).setToken(authenticator.getToken());
-                    // TODO: refactor buildSessionInfo
-                    session.buildSessionInfo(null);
+                    HttpClient httpClient = DavGatewayHttpClientFacade.getInstance(authenticator.getExchangeUri().toString());
+                    DavGatewayHttpClientFacade.createMultiThreadedHttpConnectionManager(httpClient);
+                    session = new EwsExchangeSession(httpClient, authenticator.getToken(), poolKey.userName);
 
-                } else if (Settings.EWS.equals(mode)) {
+                } else if (Settings.EWS.equals(mode) || Settings.O365.equals(mode)) {
                     if (poolKey.url.toLowerCase().endsWith("/ews/exchange.asmx")) {
                         ExchangeSession.LOGGER.debug("Direct EWS authentication");
                         HttpClient httpClient = DavGatewayHttpClientFacade.getInstance(poolKey.url);
+                        DavGatewayHttpClientFacade.createMultiThreadedHttpConnectionManager(httpClient);
                         DavGatewayHttpClientFacade.setCredentials(httpClient, poolKey.userName, poolKey.password);
-                        DavGatewayHttpClientFacade.addNTLM(httpClient);
                         session = new EwsExchangeSession(httpClient, poolKey.userName);
-                        session.buildSessionInfo(null);
                     } else {
                         ExchangeSession.LOGGER.debug("OWA authentication in EWS mode");
-                        session = new EwsExchangeSession(poolKey.url, poolKey.userName, poolKey.password);
+                        ExchangeFormAuthenticator exchangeFormAuthenticator = new ExchangeFormAuthenticator();
+                        exchangeFormAuthenticator.setUrl(poolKey.url);
+                        exchangeFormAuthenticator.setUsername(poolKey.userName);
+                        exchangeFormAuthenticator.setPassword(poolKey.password);
+                        exchangeFormAuthenticator.authenticate();
+                        session = new EwsExchangeSession(exchangeFormAuthenticator.getHttpClient(),
+                                exchangeFormAuthenticator.getExchangeUri(), exchangeFormAuthenticator.getUsername());
                     }
                 } else {
+                    ExchangeFormAuthenticator exchangeFormAuthenticator = new ExchangeFormAuthenticator();
+                    exchangeFormAuthenticator.setUrl(poolKey.url);
+                    exchangeFormAuthenticator.setUsername(poolKey.userName);
+                    exchangeFormAuthenticator.setPassword(poolKey.password);
+                    exchangeFormAuthenticator.authenticate();
                     try {
-                        session = new DavExchangeSession(poolKey.url, poolKey.userName, poolKey.password);
+                        session = new DavExchangeSession(exchangeFormAuthenticator.getHttpClient(),
+                                exchangeFormAuthenticator.getExchangeUri(),
+                                exchangeFormAuthenticator.getUsername());
                     } catch (WebdavNotAvailableException e) {
                         if (Settings.AUTO.equals(mode)) {
                             ExchangeSession.LOGGER.debug(e.getMessage() + ", retry with EWS");
-                            session = new EwsExchangeSession(poolKey.url, poolKey.userName, poolKey.password);
+                            session = new EwsExchangeSession(exchangeFormAuthenticator.getHttpClient(),
+                                    exchangeFormAuthenticator.getExchangeUri(),
+                                    exchangeFormAuthenticator.getUsername());
                         } else {
                             throw e;
                         }
                     }
                 }
+                checkWhiteList(session.getEmail());
                 ExchangeSession.LOGGER.debug("Created new session " + session + " for user " + poolKey.userName);
             }
             // successful login, put session in cache
@@ -222,6 +245,27 @@ public final class ExchangeSessionFactory {
         return session;
     }
 
+    /**
+     * Check if whitelist is empty or email is allowed.
+     * userWhiteList is a comma separated list of values.
+     * \@company.com means all domain users are allowed
+     * @param email user email
+     */
+    private static void checkWhiteList(String email) throws DavMailAuthenticationException {
+        String whiteListString = Settings.getProperty("davmail.userWhiteList");
+        if (whiteListString != null && !whiteListString.isEmpty()) {
+            for (String whiteListvalue : whiteListString.split(",")) {
+                if (whiteListvalue.startsWith("@") && email.endsWith(whiteListvalue)) {
+                    return;
+                } else if (email.equalsIgnoreCase(whiteListvalue)) {
+                    return;
+                }
+            }
+            ExchangeSession.LOGGER.warn(email + " not allowed by whitelist");
+            throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
+        }
+    }
+
     /**
      * Get a non expired session.
      * If the current session is not expired, return current session, else try to create a new session
@@ -341,7 +385,6 @@ public final class ExchangeSessionFactory {
             if (enumeration != null) {
                 while (!up && enumeration.hasMoreElements()) {
                     NetworkInterface networkInterface = enumeration.nextElement();
-                    //noinspection Since15
                     up = networkInterface.isUp() && !networkInterface.isLoopback()
                             && networkInterface.getInetAddresses().hasMoreElements();
                 }
