@@ -27,13 +27,15 @@ import davmail.exchange.ews.GetFolderMethod;
 import davmail.exchange.ews.GetUserConfigurationMethod;
 import davmail.http.DavGatewayHttpClientFacade;
 import org.apache.commons.httpclient.HttpClient;
-import org.apache.commons.httpclient.util.URIUtil;
+import org.apache.http.client.utils.URIBuilder;
 import org.apache.log4j.Logger;
 
 import javax.swing.*;
 import java.io.IOException;
 import java.net.Authenticator;
 import java.net.PasswordAuthentication;
+import java.net.URI;
+import java.net.URISyntaxException;
 
 public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
 
@@ -45,8 +47,7 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
     String code = null;
 
     String resource = "https://outlook.office365.com";
-    String ewsUrl = resource + "/EWS/Exchange.asmx";
-    String authorizeUrl = "https://login.microsoftonline.com/common/oauth2/authorize";
+    URI ewsUrl = URI.create(resource + "/EWS/Exchange.asmx");
 
     private O365InteractiveAuthenticatorFrame o365InteractiveAuthenticatorFrame;
 
@@ -59,7 +60,7 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
     }
 
     @Override
-    public String getEWSUrl() {
+    public URI getExchangeUri() {
         return ewsUrl;
     }
 
@@ -77,18 +78,35 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
 
 
     public void authenticate() throws IOException {
+        // allow cross domain requests for Okta form support
+        System.setProperty("sun.net.http.allowRestrictedHeaders", "true");
+
         // common DavMail client id
         final String clientId = Settings.getProperty("davmail.oauth.clientId", "facd6cff-a294-4415-b59f-c5b01937d7bd");
         // standard native app redirectUri
         final String redirectUri = Settings.getProperty("davmail.oauth.redirectUri", "https://login.microsoftonline.com/common/oauth2/nativeclient");
+        // company tenantId or common
+        String tenantId = Settings.getProperty("davmail.oauth.tenantId", "common");
 
-        final String initUrl = authorizeUrl
-                + "?client_id=" + clientId
-                + "&response_type=code"
-                + "&redirect_uri=" + redirectUri
-                + "&response_mode=query"
-                + "&resource=" + resource
-                + "&login_hint=" + URIUtil.encodeWithinQuery(username);
+        URI uri;
+        try {
+            uri = new URIBuilder()
+                    .setScheme("https")
+                    .setHost("login.microsoftonline.com")
+                    .setPath("/"+tenantId+"/oauth2/authorize")
+                    .addParameter("client_id", clientId)
+                    .addParameter("response_type", "code")
+                    .addParameter("redirect_uri", redirectUri)
+                    .addParameter("response_mode", "query")
+                    .addParameter("resource", resource)
+                    .addParameter("login_hint", username)
+                    // force consent
+                    //.addParameter("prompt", "consent")
+                    .build();
+        } catch (URISyntaxException e) {
+            throw new IOException(e);
+        }
+        final String initUrl = uri.toString();
 
         // set default authenticator
         Authenticator.setDefault(new Authenticator() {
@@ -119,7 +137,13 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
                     o365InteractiveAuthenticatorFrame.setO365InteractiveAuthenticator(O365InteractiveAuthenticator.this);
                     o365InteractiveAuthenticatorFrame.authenticate(initUrl, redirectUri);
                 } catch (NoClassDefFoundError e) {
-                    errorCode = "Unable to load JavaFX (OpenJFX)";
+                    LOGGER.warn("Unable to load JavaFX (OpenJFX), switch to manual mode");
+                    O365ManualAuthenticatorDialog o365ManualAuthenticatorDialog = new O365ManualAuthenticatorDialog(initUrl);
+                    code = o365ManualAuthenticatorDialog.getCode();
+                    isAuthenticated = code != null;
+                    if (!isAuthenticated) {
+                        errorCode = "User did not provide authentication code";
+                    }
                 }
             }
         });
@@ -143,7 +167,7 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
         }
 
         if (isAuthenticated) {
-            token = new O365Token(clientId, redirectUri, code);
+            token = new O365Token(tenantId, clientId, redirectUri, code);
 
             LOGGER.debug("Authenticated username: " + token.getUsername());
             if (username != null && !username.isEmpty() && !username.equalsIgnoreCase(token.getUsername())) {
@@ -166,7 +190,7 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
             authenticator.authenticate();
 
             // switch to EWS url
-            HttpClient httpClient = DavGatewayHttpClientFacade.getInstance(authenticator.ewsUrl);
+            HttpClient httpClient = DavGatewayHttpClientFacade.getInstance(authenticator.ewsUrl.toString());
 
             GetFolderMethod checkMethod = new GetFolderMethod(BaseShape.ID_ONLY, DistinguishedFolderId.getInstance(null, DistinguishedFolderId.Name.root), null);
             checkMethod.setRequestHeader("Authorization", "Bearer " + authenticator.getToken().getAccessToken());
