@@ -20,48 +20,54 @@
 package davmail.exchange.auth;
 
 import davmail.Settings;
-import davmail.http.DavGatewayHttpClientFacade;
-import davmail.http.RestMethod;
+import davmail.http.HttpClientAdapter;
+import davmail.http.request.RestRequest;
 import davmail.util.IOUtil;
-import org.apache.commons.httpclient.HttpClient;
-import org.apache.commons.httpclient.SimpleHttpConnectionManager;
+import org.apache.http.Consts;
+import org.apache.http.NameValuePair;
+import org.apache.http.client.entity.UrlEncodedFormEntity;
+import org.apache.http.message.BasicNameValuePair;
 import org.apache.log4j.Logger;
 import org.codehaus.jettison.json.JSONException;
 import org.codehaus.jettison.json.JSONObject;
 
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Date;
 
 public class O365Token {
-    protected final String TOKEN_URL = "https://login.microsoftonline.com/common/oauth2/token";
     protected final String RESOURCE_URL = "https://outlook.office365.com/";
 
     protected static final Logger LOGGER = Logger.getLogger(O365Token.class);
 
     private String clientId;
+    private String tokenUrl;
     private String redirectUri;
     private String username;
     private String refreshToken;
     private String accessToken;
-    private long expireson;
+    private long expiresOn;
 
-    public O365Token(String clientId, String redirectUri) {
+    public O365Token(String tenantId, String clientId, String redirectUri) {
         this.clientId = clientId;
         this.redirectUri = redirectUri;
+        this.tokenUrl = "https://login.microsoftonline.com/"+tenantId+"/oauth2/token";
     }
 
-    public O365Token(String clientId, String redirectUri, String code) throws IOException {
+    public O365Token(String tenantId, String clientId, String redirectUri, String code) throws IOException {
         this.clientId = clientId;
         this.redirectUri = redirectUri;
+        this.tokenUrl = "https://login.microsoftonline.com/"+tenantId+"/oauth2/token";
 
-        RestMethod tokenMethod = new RestMethod(TOKEN_URL);
-        tokenMethod.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
-        tokenMethod.addParameter("grant_type", "authorization_code");
-        tokenMethod.addParameter("code", code);
-        tokenMethod.addParameter("redirect_uri", redirectUri);
-        tokenMethod.addParameter("client_id", clientId);
+        ArrayList<NameValuePair> parameters = new ArrayList<NameValuePair>();
+        parameters.add(new BasicNameValuePair("grant_type", "authorization_code"));
+        parameters.add(new BasicNameValuePair("code", code));
+        parameters.add(new BasicNameValuePair("redirect_uri", redirectUri));
+        parameters.add(new BasicNameValuePair("client_id", clientId));
 
-        executeMethod(tokenMethod);
+        RestRequest tokenRequest = new RestRequest(tokenUrl, new UrlEncodedFormEntity(parameters, Consts.UTF_8));
+
+        executeRequest(tokenRequest);
     }
 
 
@@ -72,16 +78,16 @@ public class O365Token {
     public void setJsonToken(JSONObject jsonToken) throws IOException {
         try {
             if (jsonToken.opt("error") != null) {
-                throw new IOException(jsonToken.optString("error")+" "+jsonToken.optString("error_description"));
+                throw new IOException(jsonToken.optString("error") + " " + jsonToken.optString("error_description"));
             }
             // access token expires after one hour
             accessToken = jsonToken.getString("access_token");
             // precious refresh token
             refreshToken = jsonToken.getString("refresh_token");
             // expires_on is in second, not millisecond
-            expireson = jsonToken.getLong("expires_on") * 1000;
+            expiresOn = jsonToken.getLong("expires_on") * 1000;
 
-            LOGGER.debug("Access token expires at " + new Date(expireson) + ": " + accessToken);
+            LOGGER.debug("Access token expires " + new Date(expiresOn));
 
             String decodedBearer = IOUtil.decodeBase64AsString(accessToken.substring(accessToken.indexOf('.') + 1, accessToken.lastIndexOf('.')) + "==");
             JSONObject tokenBody = new JSONObject(decodedBearer);
@@ -89,7 +95,7 @@ public class O365Token {
             username = tokenBody.getString("unique_name");
 
             if (Settings.getBooleanProperty("davmail.oauth.persistToken", false)) {
-                Settings.setProperty("davmail.oauth."+username.toLowerCase()+".refreshToken", refreshToken);
+                Settings.setProperty("davmail.oauth." + username.toLowerCase() + ".refreshToken", refreshToken);
                 Settings.save();
             }
         } catch (JSONException e) {
@@ -107,45 +113,42 @@ public class O365Token {
 
     public String getAccessToken() throws IOException {
         // detect expiration and refresh token
-        if (System.currentTimeMillis() > expireson - 60000) {
-            LOGGER.info("Access token expires soon, trying to refresh it");
+        if (System.currentTimeMillis() > expiresOn - 60000) {
+            LOGGER.debug("Access token expires soon, trying to refresh it");
             refreshToken();
         }
-        LOGGER.info("Access token for " + username + " expires in " + (expireson - System.currentTimeMillis()) / 60000 + " minutes");
+        //LOGGER.debug("Access token for " + username + " expires in " + (expiresOn - System.currentTimeMillis()) / 60000 + " minutes");
         return accessToken;
     }
 
     public void setAccessToken(String accessToken) {
         this.accessToken = accessToken;
         // assume unexpired token
-        expireson = System.currentTimeMillis() + 1000 * 60 * 60;
+        expiresOn = System.currentTimeMillis() + 1000 * 60 * 60;
     }
 
     public void refreshToken() throws IOException {
-        RestMethod tokenMethod = new RestMethod(TOKEN_URL);
-        tokenMethod.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
-        tokenMethod.addParameter("grant_type", "refresh_token");
-        tokenMethod.addParameter("refresh_token", refreshToken);
-        tokenMethod.addParameter("redirect_uri", redirectUri);
-        tokenMethod.addParameter("client_id", clientId);
-        tokenMethod.addParameter("resource", "https://outlook.office365.com/");
+        ArrayList<NameValuePair> parameters = new ArrayList<NameValuePair>();
+        parameters.add(new BasicNameValuePair("grant_type", "refresh_token"));
+        parameters.add(new BasicNameValuePair("refresh_token", refreshToken));
+        parameters.add(new BasicNameValuePair("redirect_uri", redirectUri));
+        parameters.add(new BasicNameValuePair("client_id", clientId));
+        parameters.add(new BasicNameValuePair("resource", "https://outlook.office365.com/"));
 
-        executeMethod(tokenMethod);
+        RestRequest tokenRequest = new RestRequest(tokenUrl, new UrlEncodedFormEntity(parameters, Consts.UTF_8));
+
+        executeRequest(tokenRequest);
     }
 
-    private void executeMethod(RestMethod tokenMethod) throws IOException {
-        HttpClient httpClient = null;
+    private void executeRequest(RestRequest tokenMethod) throws IOException {
+        HttpClientAdapter httpClientAdapter = new HttpClientAdapter(RESOURCE_URL);
         try {
-            httpClient = DavGatewayHttpClientFacade.getInstance(RESOURCE_URL);
-            httpClient.executeMethod(tokenMethod);
+            httpClientAdapter.execute(tokenMethod);
             setJsonToken(tokenMethod.getJsonResponse());
 
         } finally {
-            tokenMethod.releaseConnection();
             // do not keep login connections open
-            if (httpClient != null) {
-                ((SimpleHttpConnectionManager) httpClient.getHttpConnectionManager()).shutdown();
-            }
+            httpClientAdapter.close();
         }
 
     }
