@@ -34,7 +34,6 @@ import org.apache.commons.httpclient.methods.GetMethod;
 import org.apache.commons.httpclient.params.HttpClientParams;
 import org.apache.commons.httpclient.params.HttpMethodParams;
 import org.apache.commons.httpclient.util.IdleConnectionTimeoutThread;
-import org.apache.commons.httpclient.util.URIUtil;
 import org.apache.jackrabbit.webdav.DavException;
 import org.apache.jackrabbit.webdav.MultiStatusResponse;
 import org.apache.jackrabbit.webdav.client.methods.CopyMethod;
@@ -78,12 +77,17 @@ public final class DavGatewayHttpClientFacade {
 
         Security.setProperty("ssl.SocketFactory.provider", "davmail.http.DavGatewaySSLSocketFactory");
 
+        // reenable basic proxy authentication on Java >= 1.8.111
+        System.setProperty("jdk.http.auth.tunneling.disabledSchemes", "");
+
         DavGatewayHttpClientFacade.start();
 
         // register custom cookie policy
         CookiePolicy.registerCookieSpec("DavMailCookieSpec", DavMailCookieSpec.class);
 
         AuthPolicy.registerAuthScheme(AuthPolicy.BASIC, LenientBasicScheme.class);
+        // register the jcifs based NTLMv2 implementation
+        AuthPolicy.registerAuthScheme(AuthPolicy.NTLM, NTLMv2Scheme.class);
         try {
             WORKSTATION_NAME = InetAddress.getLocalHost().getHostName();
         } catch (Throwable t) {
@@ -384,7 +388,7 @@ public final class DavGatewayHttpClientFacade {
     }
 
 
-    private static String getLocationValue(HttpMethod method) throws URIException {
+    private static String getLocationValue(HttpMethod method) {
         String locationValue = null;
         Header location = method.getResponseHeader("Location");
         if (location != null && isRedirect(method.getStatusCode())) {
@@ -627,9 +631,6 @@ public final class DavGatewayHttpClientFacade {
         // disable preemptive authentication
         httpClient.getParams().setParameter(HttpClientParams.PREEMPTIVE_AUTHENTICATION, false);
 
-        // register the jcifs based NTLMv2 implementation
-        AuthPolicy.registerAuthScheme(AuthPolicy.NTLM, NTLMv2Scheme.class);
-
         ArrayList<String> authPrefs = new ArrayList<String>();
         authPrefs.add(AuthPolicy.NTLM);
         authPrefs.add(AuthPolicy.DIGEST);
@@ -642,7 +643,7 @@ public final class DavGatewayHttpClientFacade {
         // separate domain from username in credentials
         AuthScope authScope = new AuthScope(null, -1);
         NTCredentials credentials = (NTCredentials) httpClient.getState().getCredentials(authScope);
-        if (credentials != null && credentials.getDomain() == null) {
+        if (credentials != null && (credentials.getDomain() == null || credentials.getDomain().isEmpty())) {
             setCredentials(httpClient, credentials.getUserName(), credentials.getPassword());
         }
     }
@@ -751,7 +752,7 @@ public final class DavGatewayHttpClientFacade {
     }
 
     /**
-     * Build Http Exception from methode status
+     * Build Http Exception from method status
      *
      * @param method Http Method
      * @return Http Exception
